#ifndef __CWIDGET_HPP_
#define __CWIDGET_HPP_

#include <Ph.h>
#include <Pt.h>
#include <photon/PtWidget.h>
#include <phcpp/defs.hpp>
#include <phcpp/PhTrace.h>

class CWidget
{	protected:
		PtWidget_t * pwgt;
		PtWidgetClassRef_t * pulClassRef;
		BOOL b;

	public:
		CWidget() {  pwgt = NULL;  pulClassRef = NULL; };
		CWidget(PtWidget_t * pobjParent, UINT uiNumArgs, _CONST PtArg_t * pArgs);
		virtual ~CWidget();
		SetResource(long lType, _CONST long lValue, _CONST long lLen);
		_CONST long * GetResource(long lType); 
		_CONST long * GetResource(long lType, _CONST long * plValue);

		inline Realize()
		{  if(PtRealizeWidget(this->pwgt) == -1)
		     PhTrace("CWidget :: Realize() failed.\n");
		}

		inline ReRealize()
		{  if(PtReRealizeWidget(this->pwgt) == -1)
		     PhTrace("CWidget :: ReRealize() failed.\n");
		}

		PtWidget_t * ChildBack();
		PtWidget_t * ChildFront();
		PhDim_t * Dim(PhDim_t * pdim) const;
		PhRect_t * Extent(PhRect_t * prect) const;
		long Flags() const;
		PhPoint_t * Offset(PhPoint_t * ppntOffset) const;

		inline _CONST PtWidget_t * Parent() const
		{  return(PtWidgetParent(pwgt));
		}

		inline PhRid_t Rid() const
		{  return(PtWidgetRid(pwgt));
		}

		inline PtWidget_t * Skip(_CONST PtWidget_t * pobjRoot) const
		{  return(PtWidgetSkip(pobjRoot, pwgt));
		}

		inline PhRect_t * Canvas(PhRect_t * prectCanvas) const
		{  return(PtWidgetCanvas(pwgt, prectCanvas));
		}

		inline PtWidget_t * BrotherInFront() const
		{  return(PtWidgetBrotherInFront(pwgt));
		}

		inline PtWidget_t * BrotherBehind() const
		{  return(PtWidgetBrotherBehind(pwgt));
		}

		inline PhArea_t * Area(PhArea_t * prectArea) const
		{  return(PtWidgetArea(pwgt, prectArea));
		}

		inline PtWidget_t * ValidParent() const
		{  return(PtValidParent(pwgt, pulClassRef));
		}

		inline BOOL Unrealize()
		{  return(PtUnrealizeWidget(pwgt));
		}

		inline BOOL TimerArm(unsigned uiMsec)
		{  return(PtTimerArm(pwgt, uiMsec));
		}

		inline Sync()
		{  PtSyncWidget(pwgt);
		}

		inline PtWidget_t * SetParentWidget()
		{  return(PtSetParentWidget(pwgt));
		}

		inline PhArea_t * SetAreaFromCanvas(_CONST PhRect_t * prectCanvas, PhArea_t * ptsArea)
		{  return(PtSetAreaFromWidgetCanvas(pwgt, prectCanvas, ptsArea));
		}

		inline PhArea_t * SetAreaFromExtent(_CONST PhRect_t * prectExtent, PhArea_t * ptsArea)
		{  return(PtSetAreaFromExtent(pwgt, prectExtent, ptsArea));
		}

		inline BOOL SendEvent(_CONST PhEvent_t * ptsEvent)
		{  return(PtSendEventToWidget(pwgt, ptsEvent));
		}

		inline BOOL ReParent(PtWidget_t * pobjParent)
		{  return(PtReParentWidget(pwgt, pobjParent));
		}

		inline RemoveHotkeyHandler(UINT uiKeySymCap, UINT uiKeyMods, short nFlags, _CONST void * pvData, void (*pfnCallback)())
		{  PtRemoveHotkeyHandler(pwgt, uiKeySymCap, uiKeyMods, nFlags, pvData, (void (* )(Pt_widget *, void *, Pt_callback_info *))pfnCallback);
		}

		inline RemoveEventHandlers(_CONST PtRawCallback_t * paCallbackDefs, UINT uiNumHandlers)
		{  PtRemoveEventHandlers(pwgt, paCallbackDefs, uiNumHandlers);
		}

		inline RemoveEventHandler(ULONG ulEventMask, void (*pfnCallback)(), _CONST void * pvData)
		{  PtRemoveEventHandler(pwgt, ulEventMask, (int (* )(Pt_widget *, void *, Pt_callback_info *)) pfnCallback, pvData);
		}

		inline RemoveCallbacks(ULONG ulCallbackType, _CONST PtCallback_t * paCallbackDefs, UINT uiNumCallbacks)
		{  PtRemoveCallbacks(pwgt, ulCallbackType, (Pt_callback *) paCallbackDefs, uiNumCallbacks);
		}

		inline RemoveCallback(ULONG ulCallbackType, void (*pfnCallback)(), _CONST void * pvData)
		{  PtRemoveCallback(pwgt, ulCallbackType, (void (* )(PtWidget_t *, void *, PtCallbackInfo_t *)) pfnCallback, pvData);
		}

		inline PhSysInfo_t * QuerySystemInfo(PhSysInfo_t * ptsSysInfo)
		{  return(PtQuerySystemInfo(pwgt, ptsSysInfo));
		}

		inline PhRect_t * LabelCanvas(PhRect_t * prectCanvas) const
		{  return(PtLabelWidgetCanvas(pwgt, prectCanvas));
		}

		inline int IsFocused() const
		{  return(PtIsFocused(pwgt));
		}

		inline PtWidget_t * InflateBalloon(PtWidget_t * pobjParent, int iPos, _CONST char * pszData, _CONST char * pszFont, PgColor_t ulFill, PgColor_t ulTextColor)
		{  return(PtInflateBalloon(pobjParent, pwgt, iPos, pszData, pszFont, ulFill, ulTextColor));
		}

		inline PtWidget_t * GlobalFocusPrevFrom(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusPrevFrom(pwgt, ptsEvent));
		}

		inline PtWidget_t * GlobalFocusPrevContainer(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusPrevContainer(pwgt, ptsEvent));
		}

		inline PtWidget_t * GlobalFocusPrev(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusPrev(pwgt, ptsEvent));
		}

		inline PtWidget_t * GlobalFocusNextFrom(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusNextFrom(pwgt, ptsEvent));
		}

		inline PtWidget_t * GlobalFocusNextContainer(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusNextContainer(pwgt, ptsEvent));
		}

		inline PtWidget_t * GlobalFocusNext(_CONST PhEvent_t * ptsEvent) const
		{  return(PtGlobalFocusNext(pwgt, ptsEvent));
		}

		inline PtWidget_t * GetParent(_CONST PtWidgetClassRef_t * pCParent) const
		{  return(PtGetParent(pwgt, pCParent));
		}

		inline GetAbsPosition(PhPoint_t * pntAbs) const
		{  PtGetAbsPosition(pwgt, &pntAbs->x, &pntAbs->y);
		}

		inline PtWidget_t * FindFocusChild() const
		{  return(PtFindFocusChild(pwgt));
		}

		inline PtWidget_t * FindDisjoint() const
		{  return(PtFindDisjoint(pwgt));
		}

		inline PtWidget_t * FindContainer() const
		{  return(PtFindContainer(pwgt));
		}

		inline BOOL ExtentFamily() const
		{  return(PtExtentWidgetFamily(pwgt));
		}

		inline virtual BOOL Destroy()
		{  return(PtDestroyWidget(pwgt));
		}

		inline BOOL Damage()
		{  return(PtDamageWidget(pwgt));
		}

		inline BOOL DamageExtent(_CONST PhRect_t * ptsExtent)
		{  return(PtDamageExtent(pwgt, ptsExtent));
		}

		inline ContainerGiveFocus(_CONST PhEvent_t * ptsEvent)
		{  PtContainerGiveFocus(pwgt, ptsEvent);
		}

		inline ContainerFocusPrev(_CONST PhEvent_t * ptsEvent)
		{  PtContainerFocusPrev(pwgt, ptsEvent);
		}

		inline ContainerFocusNext(_CONST PhEvent_t * ptsEvent)
		{  PtContainerFocusNext(pwgt, ptsEvent);
		}

		inline ContainerNullFocus(_CONST PhEvent_t * ptsEvent)
		{  PtContainerNullFocus(pwgt, ptsEvent);
		}

		inline PtWidget_t * ContainerFindFocus()
		{  return(PtContainerFindFocus(pwgt));
		}

		inline BOOL ClearWidget()
		{  return(PtClearWidget(pwgt));
		}

		inline BOOL CalculateExtent()
		{  return((BOOL)PtExtentWidget(pwgt));
		}

		inline int AskQuestion(_CONST char * pszTitle, _CONST char * pszQuestion, _CONST char * pszFont, _CONST char * pszBtn1, _CONST char * pszBtn2, _CONST char * pszBtn3, int iDefBtn)
		{  return(PtAskQuestion(pwgt, pszTitle, pszQuestion, pszFont, pszBtn1, pszBtn2, pszBtn3, iDefBtn));
		}

		inline AddHotkeyHandler(UINT uiKeySymCap, UINT uiKeyMods, USHORT usFlags, _CONST void * pvData, void (*pfnCallback)() )
		{  PtAddHotkeyHandler(pwgt, uiKeySymCap, uiKeyMods, usFlags, pvData, (void (* ) (Pt_widget *, void *, Pt_callback_info *)) pfnCallback);
		}

		inline AddEventHandlers(_CONST PtRawCallback_t * paCallbackDefs, UINT uiNumHandlers)
		{  PtAddEventHandlers(pwgt, paCallbackDefs, uiNumHandlers);
		}

		inline AddEventHandler(ULONG ulEventMask, void (*pfnCallback)(), _CONST void * pvData)
		{  PtAddEventHandler(pwgt, ulEventMask, (int (* )(Pt_widget *, void *, Pt_callback_info *)) pfnCallback, pvData);
		}

		inline AddCallbacks(ULONG ulCallbackType, _CONST PtCallback_t * paCallbackDefs, UINT uiNumCallbacks)
		{  PtAddCallbacks(pwgt, ulCallbackType, paCallbackDefs, uiNumCallbacks);
		}

		inline AddCallback(ULONG ulCallbackType, void (*pfnCallback)(), _CONST void * pvData)
		{  PtAddCallback(pwgt, ulCallbackType, (void (* )(PtWidget_t *, void *, PtCallbackInfo_t *)) pfnCallback, pvData);
		}

		inline const PtWidgetClassRef_t * Class() const
		{  return(pulClassRef);
		}

		operator PtWidget_t *();
		BOOL AssignWidgetPointer(PtWidget_t * pwgt);
};
#endif
